# Generated by Django 5.2.9 on 2025-12-16 15:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('tags', '0001_initial'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='tag',
            unique_together=set(),
        ),
        migrations.AddConstraint(
            model_name='tag',
            constraint=models.UniqueConstraint(fields=('slug', 'content_type'), name='unique_tag'),
        ),
    ]



from typing import Generator, Iterable

from wbcore.contrib.icons import WBIcon
from wbcore.metadata.configs.buttons.buttons import DropDownButton
from wbcore.signals.instance_buttons import (
    add_button,
    add_extra_button,
    add_instance_button,
)
from wbcore.utils.importlib import parse_signal_received_for_module

from ..base import WBCoreViewConfig
from .enums import Button


class ButtonViewConfig(WBCoreViewConfig):
    SHOW_INLINE: bool = False  # set to true if the class needs to show custom button on inline

    metadata_key = "buttons"
    config_class_attribute = "button_config_class"

    def serialize(self, base_buttons: set[Button], remote_resources):
        """
        Serialize the button view config

        Args:
            base_buttons: the base button gathered from the view config
            remote_resources: the return of the send function
        Returns:
            Yield the serialized button, without duplicates and appends the module prefix to the remote button
        """
        base_buttons = list(
            zip([None] * len(base_buttons), base_buttons, strict=False)
        )  # append an empty perfix for base buttons
        for prefix, btn in parse_signal_received_for_module(remote_resources):
            base_buttons.append((prefix, btn))

        keys = []
        for key_prefix, btn in sorted(
            filter(lambda x: x[1], base_buttons), key=lambda e: (e[1].weight, e[1].label, e[1].title)
        ):
            btn_serialized = btn.serialize(self.request, key_prefix=key_prefix)
            if "key" not in btn_serialized or btn_serialized["key"] not in keys:  # ensure we don't include duplicates
                yield btn_serialized

    # Utils
    def order_buttons(self, buttons: set, ordering: list):
        yield from filter(lambda b: b in buttons, ordering)

    # FSM Button Configuration
    FSM_LIST = True
    FSM_INSTANCE = True
    FSM_DROPDOWN = False
    FSM_DROPDOWN_ICON = WBIcon.ADD.icon
    FSM_DROPDOWN_LABEL = "Transitions"
    FSM_WEIGHT = 100

    def get_fsm_buttons(self) -> set:
        if self.FSM_DROPDOWN and (fsm_buttons := self.view.FSM_BUTTONS) and len(fsm_buttons) > 0:
            return {
                DropDownButton(
                    label=self.FSM_DROPDOWN_LABEL,
                    icon=self.FSM_DROPDOWN_ICON,
                    title=self.FSM_DROPDOWN_LABEL,
                    weight=self.FSM_WEIGHT,
                    buttons=tuple(fsm_buttons),
                )
            }
        return getattr(self.view, "FSM_BUTTONS", set())

    # Create Button Configuration
    CREATE_BUTTONS = frozenset(
        {
            Button.SAVE.value,
            Button.SAVE_AND_CLOSE.value,
            Button.SAVE_AND_NEW.value,
        }
    )
    CREATE_BUTTONS_ORDERING = [
        Button.SAVE.value,
        Button.SAVE_AND_CLOSE.value,
        Button.SAVE_AND_NEW.value,
    ]

    def get_create_buttons(self) -> Iterable:
        buttons = set(self.CREATE_BUTTONS)
        yield from self.order_buttons(buttons, self.CREATE_BUTTONS_ORDERING)

    # Custom Instance Button Configuration
    CUSTOM_INSTANCE_BUTTONS = frozenset()
    CUSTOM_LIST_INSTANCE_BUTTONS = frozenset()
    CUSTOM_EXTRA_BUTTONS = frozenset()

    def get_custom_instance_buttons(self) -> set:
        return set(self.CUSTOM_INSTANCE_BUTTONS)

    def get_custom_list_instance_buttons(self) -> set:
        return set(self.CUSTOM_LIST_INSTANCE_BUTTONS)

    def _get_custom_instance_buttons(self) -> Generator:
        custom_instance_buttons = set()
        if self.instance:
            custom_instance_buttons |= self.get_custom_instance_buttons()
            if self.FSM_INSTANCE:
                custom_instance_buttons |= self.get_fsm_buttons()

        else:
            custom_instance_buttons |= self.get_custom_list_instance_buttons()
            if self.FSM_LIST:
                custom_instance_buttons |= self.get_fsm_buttons()

        yield from self.serialize(
            custom_instance_buttons,
            add_instance_button.send(
                self.view.__class__, many=not self.instance, request=self.request, view=self.view, **self.view.kwargs
            ),
        )

    def get_custom_extra_buttons(self) -> set:
        return set(self.CUSTOM_EXTRA_BUTTONS)

    def _get_custom_extra_buttons(self):
        yield from self.serialize(
            self.get_custom_extra_buttons(),
            add_extra_button.send(
                self.view.__class__, instance=self.instance, request=self.request, view=self.view, **self.view.kwargs
            ),
        )

    # Custom Button Configuration
    CUSTOM_BUTTONS = frozenset()

    def get_custom_buttons(self) -> set:
        return set(self.CUSTOM_BUTTONS)

    def _get_custom_buttons(self):
        yield from self.serialize(
            self.get_custom_buttons(),
            add_button.send(self.view.__class__, request=self.request, view=self.view, **self.view.kwargs),
        )

    def get_metadata(self) -> dict:
        return {
            "custom_instance": self._get_custom_instance_buttons(),
            "custom": self._get_custom_buttons()
            if (not self.view.inline or self.SHOW_INLINE)
            else [],  # we do not show button for list display generated through inline
            "extra": self._get_custom_extra_buttons(),
            "save": self.get_create_buttons(),
        }



def parse_signal_received_for_module(receiver_response) -> tuple[str, any]:
    for receiver, response in receiver_response:
        if response:
            yield receiver.__module__.split(".", 1)[0], response



# Generated by Django 5.2.9 on 2025-12-16 15:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('currency', '0001_initial'),
        ('io', '0008_importsource_resource_kwargs'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='currencyfxrates',
            unique_together=set(),
        ),
        migrations.AddConstraint(
            model_name='currencyfxrates',
            constraint=models.UniqueConstraint(fields=('date', 'currency'), name='unique_currency'),
        ),
    ]



# Generated by Django 5.2.9 on 2025-12-16 15:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('io', '0008_importsource_resource_kwargs'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='importedobjectproviderrelationship',
            unique_together=set(),
        ),
        migrations.AlterUniqueTogether(
            name='parserhandler',
            unique_together=set(),
        ),
        migrations.AddConstraint(
            model_name='importedobjectproviderrelationship',
            constraint=models.UniqueConstraint(fields=('content_type', 'object_id', 'provider'), name='unique_contentobjectprovideridentifierrelationship'),
        ),
        migrations.AddConstraint(
            model_name='importedobjectproviderrelationship',
            constraint=models.UniqueConstraint(fields=('content_type', 'provider_identifier', 'provider'), name='unique_contentobjectprovideridentifier'),
        ),
        migrations.AddConstraint(
            model_name='parserhandler',
            constraint=models.UniqueConstraint(fields=('parser', 'handler'), name='unique_parserhandler'),
        ),
    ]



from django.conf import settings
from django.template import TemplateDoesNotExist
from django.template.backends.django import DjangoTemplates as BaseDjangoTemplates
from django.template.backends.django import Template as BaseTemplate
from django.template.backends.django import reraise
from django.template.context import make_context


class Template(BaseTemplate):
    def render(self, context=None, request=None):
        context = make_context(context, request, autoescape=self.backend.engine.autoescape)
        context.update(
            {
                "PLOTLY_CDN": settings.WBCORE_PLOTLY_CDN_URL,
                "FRONTEND_VERSION": settings.FRONTEND_VERSION,
            }
        )
        try:
            return self.template.render(context)
        except TemplateDoesNotExist as exc:
            reraise(exc, self.backend)


class DjangoTemplates(BaseDjangoTemplates):
    def get_template(self, template_name):
        try:
            return Template(self.engine.get_template(template_name), self)
        except TemplateDoesNotExist as exc:
            reraise(exc, self)



import enum


class Queue(enum.Enum):
    HIGH_PRIORITY = "high_priority"  # High priority (for urgent, short tasks)
    DEFAULT = "default"  # for normal tasks
    BACKGROUND = "background"  # Less urgent, medium - length tasks (few minutes)
    EXTENDED_BACKGROUND = "extended_background"  # Very long-lasting background tasks (tasks lasting hours)



# import pytest
# from django.test import Client, TestCase
# from django.urls import reverse
# from wbcore.contrib.authentication.factories import UserFactory
# from wbcore.contrib.example_app.factories import EventFactory, EventTypeFactory
# from wbcore.contrib.example_app.models import Event, EventType
# from wbcore.contrib.example_app.serializers import (
#     EventModelSerializer,
#     EventTypeModelSerializer,
# )
# from wbcore.contrib.example_app.tests.test_viewsets.test_utils_viewsets import (
#     find_instances_in_response,
#     get_create_view,
#     get_delete_view,
#     get_detail_view,
#     get_partial_view,
#     get_update_view,
# )
# from wbcore.contrib.example_app.viewsets import EventModelViewSet, EventTypeModelViewSet
#
#
# @pytest.mark.django_db
# class TestEventModelViewSet(TestCase):
#     def setUp(self) -> None:
#         self.user = UserFactory.create(is_active=True, is_superuser=True)
#         self.client = Client()
#         self.client.force_login(user=self.user)
#         self.list_url = reverse("example_app:event-list")
#         self.detail_url_str = "example_app:event-detail"
#
#     def test_list_view(self):
#         response = self.client.get(self.list_url)
#         print(response.content)
#         self.assertEqual(response.status_code, 200)
#
#     def test_create_view(self):
#         event = EventFactory.create()
#         response = get_create_view(self.client, event, self.user, self.list_url, EventModelViewSet)
#         print(response.content)
#         self.assertEqual(response.status_code, 201)
#         self.assertTrue(
#             Event.objects.filter(event_type=event.event_type, match=event.match, minute=event.minute).exists()
#         )
#
#     def test_detail_view(self):
#         instance = EventFactory.create()
#         response = get_detail_view(self.client, instance.pk, self.detail_url_str)
#         print(response.content)
#         self.assertEqual(response.status_code, 200)
#         self.assertEqual(response.data["instance"]["id"], instance.id)
#
#     def test_update_view(self):
#         instance = EventFactory.create()
#         max_match_duration = instance.match.sport.match_duration
#         instance.minute = max_match_duration - instance.minute
#         response = get_update_view(self.client, instance, EventModelSerializer, self.detail_url_str)
#         print(response.content)
#         instance.refresh_from_db()
#         self.assertEqual(response.status_code, 200)
#         self.assertEqual(response.data["instance"]["minute"], instance.minute)
#
#     def test_partial_update_view(self):
#         instance = EventFactory.create()
#         max_match_duration = instance.match.sport.match_duration
#         response = get_partial_view(
#             self.client, instance.id, {"minute": max_match_duration - instance.minute}, self.detail_url_str
#         )
#         print(response.content)
#         instance.refresh_from_db()
#         self.assertEqual(response.status_code, 200)
#         self.assertEqual(response.data["instance"]["minute"], instance.minute)
#
#     def test_delete_view(self):
#         instance = EventFactory.create()
#         response = get_delete_view(self.client, self.detail_url_str, instance.pk)
#         # It is not possible to delete an event, since the get_endpoint_url returns None. I suppose that is expected behavior.
#         self.assertEqual(response.status_code, 405)
#         self.assertTrue(Event.objects.filter(pk=instance.pk).exists())
#
#     def test_ordering_fields(self):
#         event1 = EventFactory.create(minute=30)
#         EventFactory.create(person=event1.person, minute=20, event_type=event1.event_type, match=event1.match)
#         EventFactory.create(person=event1.person, minute=40, event_type=event1.event_type, match=event1.match)
#
#         response = self.client.get(self.list_url)
#         self.assertEqual(response.status_code, 200)
#         self.assertEqual(response.data["count"], Event.objects.count())
#         self.assertEqual(response.data["results"][0]["minute"], 20)
#         self.assertEqual(response.data["results"][1]["minute"], 30)
#         self.assertEqual(response.data["results"][2]["minute"], 40)
#
#
# @pytest.mark.django_db
# class TestEventTypeModelViewSet(TestCase):
#     def setUp(self) -> None:
#         self.user = UserFactory.create(is_active=True, is_superuser=True)
#         self.client = Client()
#         self.client.force_login(user=self.user)
#         self.list_url = reverse("example_app:eventtype-list")
#         self.detail_url_str = "example_app:eventtype-detail"
#
#     def test_list_view(self):
#         response = self.client.get(self.list_url)
#         self.assertEqual(response.status_code, 200)
#
#     def test_create_view(self):
#         event_type = EventTypeFactory.create()
#         response = get_create_view(self.client, event_type, self.user, self.list_url, EventTypeModelViewSet)
#         self.assertEqual(response.status_code, 201)
#         self.assertTrue(EventType.objects.filter(name=event_type.name).exists())
#
#     def test_detail_view(self):
#         instance = EventTypeFactory.create()
#         response = get_detail_view(self.client, instance.pk, self.detail_url_str)
#         self.assertEqual(response.status_code, 200)
#         self.assertEqual(response.data["instance"]["id"], instance.id)
#
#     def test_update_view(self):
#         instance = EventTypeFactory.create()
#         instance.name = "Updated Instance"
#         response = get_update_view(self.client, instance, EventTypeModelSerializer, self.detail_url_str)
#         instance.refresh_from_db()
#         self.assertEqual(response.status_code, 200)
#         self.assertEqual(response.data["instance"]["name"], instance.name)
#
#     def test_partial_update_view(self):
#         instance = EventTypeFactory.create()
#         response = get_partial_view(self.client, instance.id, {"name": "Updated Instance"}, self.detail_url_str)
#         instance.refresh_from_db()
#         self.assertEqual(response.status_code, 200)
#         self.assertEqual(response.data["instance"]["name"], instance.name)
#
#     def test_delete_view(self):
#         instance = EventTypeFactory.create()
#         response = get_delete_view(self.client, self.detail_url_str, instance.pk)
#         self.assertEqual(response.status_code, 204)
#         self.assertFalse(Event.objects.filter(pk=instance.pk).exists())
#
#     def test_ordering_fields(self):
#         event_a = EventTypeFactory.create(name="BBB")
#         event_b = EventTypeFactory.create(name="AAA", sport=event_a.sport)
#         event_c = EventTypeFactory.create(name="CCC", sport=event_a.sport)
#
#         response = self.client.get(self.list_url)
#         self.assertEqual(response.status_code, 200)
#         self.assertEqual(response.data["count"], EventType.objects.count())
#         self.assertEqual(response.data["results"][0]["id"], event_b.id)
#         self.assertEqual(response.data["results"][1]["id"], event_a.id)
#         self.assertEqual(response.data["results"][2]["id"], event_c.id)
#
#     def test_event_type_sport(self):
#         type_a = EventTypeFactory.create()
#         type_b = EventTypeFactory.create(sport=type_a.sport)
#         type_c = EventTypeFactory.create()
#         expected_number_of_types = EventType.objects.filter(sport=type_a.sport).count()
#         event_type_url = reverse("example_app:eventtype-sport-list", args=[type_a.sport.id])
#         response = self.client.get(event_type_url)
#         type_a_found, type_b_found, type_c_found = find_instances_in_response([type_a, type_b, type_c], response)
#         self.assertEqual(response.status_code, 200)
#         self.assertEqual(
#             response.data["count"],
#             expected_number_of_types,
#             f"The answer should contain {expected_number_of_types} types",
#         )
#         self.assertTrue(type_a_found, "Type A was not found in Response")
#         self.assertTrue(type_b_found, "Type B was not found in Response")
#         self.assertFalse(type_c_found, "type C was found in Response, but should not be found")



from wbcore.configs.registry import ConfigRegistry


def test_profile_config(config_registry: ConfigRegistry, api_request):
    profile = config_registry.get_config_dict(api_request)["profile"]
    assert profile



from contextlib import suppress
from importlib import import_module
from inspect import getmembers
from typing import Any, Iterable

from django.conf import settings
from rest_framework.request import Request


class ConfigRegistry:
    """
    A class that is able to discover all functions that are wrapped in the `@register_config` decorator. This registry
    should never be used from the outside and its sole purpose is to gather all functions, so they are returned
    in the APIView of this module.
    """

    def __init__(self, *args, **kwargs):
        self.config_members = []
        self._load_configs()

    def _load_configs(self):
        for app in settings.INSTALLED_APPS:
            with suppress(ModuleNotFoundError):
                module = import_module(f"{app}.configs")
                for member in getmembers(module, lambda member: hasattr(member, "_is_config")):
                    self.config_members.append(member[1])

    def get_configs(self, request: Request) -> Iterable[tuple[str, Any]]:
        for member in self.config_members:
            if res := member(request=request):
                yield res

    def get_config_dict(self, request: Request) -> dict[str, Any]:
        return dict(self.get_configs(request))


config_registry = ConfigRegistry()



from wbcore.configs.registry import ConfigRegistry


def test_notifications_config(config_registry: ConfigRegistry, api_request):
    notifications = config_registry.get_config_dict(api_request)["notifications"]
    assert notifications["endpoint"]
    assert notifications["token"]



from wbcore.configs.registry import ConfigRegistry


def test_workflow_config(config_registry: ConfigRegistry, api_request):
    workflow = config_registry.get_config_dict(api_request)["workflow"]
    assert workflow["endpoint"]



# Generated by Django 5.2.9 on 2025-12-09 08:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('directory', '0013_alter_clientmanagerrelationship_options'),
    ]

    operations = [
        migrations.AlterField(
            model_name='entry',
            name='relationship_managers',
            field=models.ManyToManyField(blank=True, help_text='People in charge of this entry', related_name='clients', through='directory.ClientManagerRelationship', through_fields=('client', 'relationship_manager'), to='directory.person', verbose_name='Relationship Managers'),
        ),
        migrations.AlterField(
            model_name='entry',
            name='relationships',
            field=models.ManyToManyField(blank=True, through='directory.Relationship', through_fields=('from_entry', 'to_entry'), to='directory.entry', verbose_name="The Entry's Relationships"),
        ),
        migrations.AlterField(
            model_name='person',
            name='employers',
            field=models.ManyToManyField(blank=True, help_text="The person's employers", related_name='employees', through='directory.EmployerEmployeeRelationship', through_fields=('employee', 'employer'), to='directory.company', verbose_name='Employers'),
        ),
    ]



# Generated by Django 5.2.9 on 2025-12-16 15:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('authentication', '0006_auto_20231206_1422'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='token',
            unique_together=set(),
        ),
        migrations.AddConstraint(
            model_name='token',
            constraint=models.UniqueConstraint(fields=('user', 'protected_view_name'), name='unique_token'),
        ),
    ]



# Generated by Django 5.2.9 on 2025-12-09 08:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('geography', '0001_initial'),
    ]

    operations = [
        migrations.AddIndex(
            model_name='geography',
            index=models.Index(fields=['tree_id', 'lft'], name='geography_geography_tree_i739a'),
        ),
    ]



from wbcore.configs.registry import ConfigRegistry


def test_authentication_config(config_registry: ConfigRegistry, api_request):
    authentication = config_registry.get_config_dict(api_request)["authentication"]
    assert authentication



from datetime import date, timedelta

from celery import shared_task
from django.db import connection
from django.utils import timezone
from dynamic_preferences.registries import global_preferences_registry

from wbcore.workers import Queue


@shared_task(queue=Queue.BACKGROUND.value)
def clean_up_import_source(today: date | None = None):
    if not today:
        today = timezone.now()
    global_preferences = global_preferences_registry.manager()
    retention_period = global_preferences["io__import_source_retention_period"]
    retention_until = today - timedelta(days=retention_period)
    with connection.cursor() as cursor:
        cursor.execute(
            """
            UPDATE io_importsource
            SET data = '{}'::jsonb,
                log = ''
            WHERE created < %s;
        """,
            (retention_until,),
        )
